package eu.transkribus.swt_gui.htr;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.ExpandableComposite;

import eu.transkribus.core.model.beans.CITlabLaTrainConfig;
import eu.transkribus.core.model.beans.rest.ParameterMap;
import eu.transkribus.core.util.ModelUtil;
import eu.transkribus.swt.util.Fonts;
import eu.transkribus.swt.util.LabeledText;
import eu.transkribus.swt.util.SWTUtil;
import eu.transkribus.swt_gui.mainwidget.storage.Storage;
import eu.transkribus.swt_gui.models.ModelChooserButton;

public class CITlabLATrainingConfComposite extends Composite {
	private static final boolean SHOW_MODEL_TYPE = false;
	
	private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory
			.getLogger(CITlabLATrainingConfComposite.class);
	
	private Text numEpochsTxt;
	private Text learningReateTxt;
	private Combo modelTypeCombo;
	private ModelChooserButton baseModelBtn;
	
	private ExpandableComposite advancedParamsExp;
	private LabeledText scaleMin, scaleMax, scaleVal;
	private Button affineTr;
	
	public static final String[] MODEL_TYPE_ITEMS =  {"u", "ru", "aru", "laru"};

	public CITlabLATrainingConfComposite(Composite parent, int style) {
		super(parent, style);
		setLayout(new GridLayout(2, false));
		
		Label numEpochsLbl = new Label(this, SWT.NONE);
		numEpochsLbl.setText("Nr. of Epochs:");
		numEpochsTxt = new Text(this, SWT.BORDER);
		numEpochsTxt.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		numEpochsTxt.setToolTipText("The number of epochs");
		
		Label learningRateLbl = new Label(this, SWT.NONE);
		learningRateLbl.setText("Learning rate:");		
		learningReateTxt = new Text(this, SWT.BORDER);
		learningReateTxt.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		learningReateTxt.setToolTipText("The learning rate");
		
		Label baseModelLbl = new Label(this, SWT.NONE);
		baseModelLbl.setText("Base Model:");
		baseModelBtn = new ModelChooserButton(this, true, ModelUtil.TYPE_LAYOUT, getProvider());
		baseModelBtn.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));		
		
		if (SHOW_MODEL_TYPE) {
			Label modelTypeLbl = new Label(this, SWT.NONE);
			modelTypeLbl.setText("Model type:");			
			modelTypeCombo = new Combo(this, SWT.READ_ONLY);
			modelTypeCombo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
			modelTypeCombo.setToolTipText("The model type - cf. paper: https://arxiv.org/pdf/1802.03345v2.pdf");
			modelTypeCombo.setItems(MODEL_TYPE_ITEMS);
		}
		
		if (Storage.i().isAdminLoggedIn()) {
			initAdvancedParamsComposite(this);
		}
		
//		Button paperLinkBtn = new Button(this, 0);
//		paperLinkBtn.setText("Link to paper...");
//		paperLinkBtn.setImage(Images.SCRIPT_ICON);
//		SWTUtil.onSelectionEvent(paperLinkBtn, e -> {
//			org.eclipse.swt.program.Program.launch("https://arxiv.org/pdf/1802.03345v2.pdf");
//		});
		
//		Button codeLinkBtn = new Button(this, 0);
//		codeLinkBtn.setText("Link to GitHub repo...");
//		codeLinkBtn.setImage(Images.SCRIPT_CODE_ICON);
//		SWTUtil.onSelectionEvent(codeLinkBtn, e -> {
//			org.eclipse.swt.program.Program.launch("https://github.com/TobiasGruening/ARU-Net");
//		});		
		
		setDefault();
	}
	
	private void initAdvancedParamsComposite(Composite container) {
		advancedParamsExp = new ExpandableComposite(container, ExpandableComposite.COMPACT);
//		advancedParamsExp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		advancedParamsExp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));	
		Composite grp = new Composite(advancedParamsExp, SWT.SHADOW_ETCHED_IN);
		grp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		grp.setLayout(new GridLayout(1, false));
		
//		LabeledText batchSize = new LabeledText(grp, "Batch Size: ");
//		batchSize.setText("1");
		
		scaleMin = new LabeledText(grp, "Scale-Min: ");
		scaleMin.setText("0.2");
		scaleMin.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		scaleMin.setToolTipText("Minimal scaling parameter during data augmentation");
		
		scaleMax = new LabeledText(grp, "Scale-Max: ");
		scaleMax.setText("0.5");
		scaleMax.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		scaleMax.setToolTipText("Maximal scaling parameter during data augmentation");
		
		scaleVal = new LabeledText(grp, "Scale-Val: ");
		scaleVal.setText("0.33");
		scaleVal.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		scaleVal.setToolTipText("Scaling parameter used for validation");
		
		affineTr = new Button(grp, SWT.CHECK);
		affineTr.setText("Perform affine transformations");
		affineTr.setSelection(true);
		affineTr.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		affineTr.setToolTipText("Perform affine transformations for data augmentation?");
		
		advancedParamsExp.setClient(grp);
		advancedParamsExp.setText("Advanced Settings");
		advancedParamsExp.setExpanded(false);
		
		Fonts.setBoldFont(advancedParamsExp);
		advancedParamsExp.addExpansionListener(new ExpansionAdapter() {
			public void expansionStateChanged(ExpansionEvent e) {
				container.layout();
			}
		});
		
	}
	
	public void setDefault() {
		CITlabLaTrainConfig c = new CITlabLaTrainConfig();
		SWTUtil.set(numEpochsTxt, ""+c.getNumEpochs());
		SWTUtil.set(learningReateTxt, ""+c.getLearningRate());
		if (modelTypeCombo!=null) {
			modelTypeCombo.select(Arrays.asList(MODEL_TYPE_ITEMS).indexOf(c.getModelType()));
		}
		if(baseModelBtn != null) {
			baseModelBtn.setModel(null);
		}		
	}
	
	public String getProvider() {
		return ModelUtil.PROVIDER_CITLAB_BASELINES;
	}

	public List<String> validateParameters(ArrayList<String> errorList) {
		if(errorList == null) {
			errorList = new ArrayList<>();
		}
		if (!StringUtils.isNumeric(numEpochsTxt.getText())) {
			errorList.add("Number of Epochs must be a number!");
		}
		
		return errorList;
	}

	public CITlabLaTrainConfig addParameters(CITlabLaTrainConfig conf) {
		conf.getModelMetadata().setProvider(getProvider());
		conf.setNumEpochs(Integer.parseInt(numEpochsTxt.getText()));
		conf.getModelMetadata().setBaseModelId(baseModelBtn.getModelId());
		if (modelTypeCombo!=null) {
			conf.setModelType(modelTypeCombo.getText());
		}
		if (advancedParamsExp!=null && advancedParamsExp.isExpanded()) {
			logger.debug("adding advanced parameters...");
			ParameterMap advancedParams = new ParameterMap();
			advancedParams.addParameter("scale_min", scaleMin.getText());
			advancedParams.addParameter("scale_max", scaleMax.getText());
			advancedParams.addParameter("scale_val", scaleVal.getText());
			advancedParams.addBoolParam("affine_tr", affineTr.getSelection());
			conf.setAdvancedParams(advancedParams);
		}
		
		return conf;
	}

}
